package fr.velossity.sample.device.impl;

import java.util.Hashtable;
import java.util.Random;

import org.osgi.framework.BundleContext;
import org.osgi.service.wireadmin.Envelope;
import org.osgi.service.wireadmin.Producer;
import org.osgi.service.wireadmin.Wire;
import org.osgi.service.wireadmin.WireConstants;

import fr.velossity.sample.device.Device;
import fr.velossity.sample.device.Measurement;

/**
 * 
 * @author C. Saint-Marcel
 * 
 */
public class MockDeviceImpl implements Device, Producer, Runnable {

	private String identifier;

	private Thread measureGenerator;

	private Measurement lastMeasurement = null;

	private Wire[] wires;

	public Wire[] getWires() {
		return wires;
	}

	private synchronized void setWires(Wire[] wires) {
		this.wires = wires;
	}

	/*
	 * Default constructor.
	 */
	public MockDeviceImpl(String identifier, BundleContext context) {
		super();
		this.identifier = identifier;
	    Hashtable props = new Hashtable();
	    //the producer will send Measurement data (flavors)
	    Class[] flavors = new Class[] {Envelope.class};
	    props.put(WireConstants.WIREADMIN_PRODUCER_FLAVORS, flavors);
	    //the producer PID property
	    props.put("service.pid", identifier);

		// PUBLISHES the Producer service
	    context.registerService(Producer.class.getName(), this, props);   

	}

	public String getIdentifier() {
		return identifier;
	}

	public String getType() {
		return "Mock";
	}

	/**
	 * Mock device is always functional
	 */
	public String getState() {
		return DEVICE_STATE_FUNCTIONAL;
	}

	public Measurement getLastMeasure() {
		return lastMeasurement;
	}

	public synchronized void activate() {
		if (measureGenerator == null) {
			measureGenerator = new Thread(this);
			measureGenerator.start();
		}
	}

	public synchronized void deactivate() {
		measureGenerator = null;
	}
	
    /** 
     * If there are connected wires, updates them with the produced values.
     * @see Producer#consumersConnected(Wire[])
     */
	public void consumersConnected(Wire[] wires) {
		setWires(wires);
	}

	/**
	 *  This method is responsible for creating the output.
	 *  @see Producer#polled(Wire)
	 */
    public Object polled(Wire wire) {
	    return getLastMeasure();    
	}


	public void run() {
		Thread mg = measureGenerator;
		while (measureGenerator == mg) {
			lastMeasurement = new Measurement(MockDeviceImpl.this, new Random()
					.nextFloat() * 1000, "Watt", System.currentTimeMillis());
			synchronized (this) {
				if (wires != null) {
					for (int i = 0; i < wires.length; i++) {
						wires[i].update(lastMeasurement);
					}
				}
				try {
					wait(10000);
				} catch (InterruptedException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}

		}
	}
}
